@@ -25,9 +25,14 @@ class Button : public QPushButton {
     Q_OBJECT
 
 public:
-    explicit Button(QWidget *parent = 0, int maxWidth = 70, int maxHeight = 70);
-    explicit Button(const QString &text, QWidget *parent = 0, const int maxWidth = 70, const int maxHeight = 70);
-    explicit Button(const QIcon &icon, const QString &text, QWidget *parent = 0, const int maxWidth = 70, const int maxHeight = 70);
+    explicit Button(const QString &name, QWidget *parent = 0, const int maxWidth = 70, const int maxHeight = 70);
+
+public slots:
+    void swapIcon();
+
+protected:
+    QIcon icons[2];
+    int iconIndex;
 };
 
 #endif // BUTTON_H
